@@ -22,7 +22,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS); # Exporter vars
   file_modification_time getfile getmakefilelist get_verbatim_file
   getinclude include
   last_update putfile read_records read_starfish_conf
-  sfish_add_tag starfish_cmd ) ] );
+  sfish_add_tag sfish_ignore_outer starfish_cmd ) ] );
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 @EXPORT = @{ $EXPORT_TAGS{'all'} };
 
@@ -30,7 +30,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS); # Exporter vars
 use vars qw($NAME $ABSTRACT $VERSION);
 $NAME     = 'Starfish';
 $ABSTRACT = 'Perl-based System for Text-Embedded Programming and Preprocessing';
-$VERSION  = '1.23';
+$VERSION  = '1.24';
 
 use vars qw($Revision);
 ($Revision = substr(q$Revision: 392 $, 10)) =~ s/\s+$//;
@@ -66,6 +66,7 @@ sub new($@) {
 	$self->{Style}           = $::Star->{Style};
 	$self->{CodePreparation} = $::Star->{CodePreparation};
 	$self->{LineComment}     = $::Star->{LineComment};
+	$self->{IgnoreOuter}     = $::Star->{IgnoreOuter};
 	$self->{hook}            = [ @{ $::Star->{hook} } ];
     }
 
@@ -240,24 +241,24 @@ sub digest {
     $self->{Out} = '';
     $self->scan();
     while ($self->{ttype} != -1) {
-	if ($self->{ttype} > -1) {
-	    my $hook = $self->{hook}->[$self->{ttype}];
-	    if ($hook->{ht} eq 'regex') {
-		$self->{Out} .= $self->eval_dispatch;
-	    }
-	    # old style regex evaluator, should be removed, but first
-	    # fix problems with python style
-	    elsif ( @{$self->{args}} ) {
-		$self->{Out}.= &{$self->{hook}->[$self->{ttype}]->{replace}}
-		( $self, @{ $self->{args} } );
-	    }
-	    else {
-		$self->{Out}.= &{$self->{hook}->[$self->{ttype}]->{f}}
-		( $self, $self->{prefix}, $self->{currenttoken}, $self->{suffix});
-	    }
-	} # else $ttype < -1
-	else { $self->{Out}.=$self->{currenttoken} }
-	$self->scan();
+      if ($self->{ttype} > -1) {
+	my $hook = $self->{hook}->[$self->{ttype}];
+	if ($hook->{ht} eq 'regex') {
+	  $self->{Out} .= $self->eval_dispatch;
+	}
+	# old style regex evaluator, should be removed, but first
+	# fix problems with python style
+	elsif ( @{$self->{args}} ) {
+	  $self->{Out}.= &{$self->{hook}->[$self->{ttype}]->{replace}}
+	    ( $self, @{ $self->{args} } );
+	}
+	else {
+	  $self->{Out}.= &{$self->{hook}->[$self->{ttype}]->{f}}
+	    ( $self, $self->{prefix}, $self->{currenttoken}, $self->{suffix});
+	}
+      } # else $ttype < -1 (outer text)
+      else { $self->_process_outer( $self->{currenttoken} ) }
+      $self->scan();
     }
     
     if ($self->{CurrentLoop} < $self->{Loops}) {
@@ -287,6 +288,15 @@ sub digest {
     }
 }
 
+# process outer text; by default, it should be appended to $self->{Out}
+sub _process_outer {
+  my $self = shift; my $outer = shift;
+  if ($self->{REPLACE} && $self->{IgnoreOuter}) {  }
+  else { $self->{Out} .= $outer }
+  # Comment: something should be done if IgnoreOuter is true and
+  # not in REPLACE mode; maybe comment-out outer text? (todo?)
+}
+
 sub _index {
     my $str = shift;
     my $subs = shift;
@@ -298,7 +308,7 @@ sub _index {
 }
 
 # $self->{ttype}: -1 EOF
-#             -2 outer text
+#             -2 outer text (but also handled directly)
 sub scan {
     my $self = shift;
 
@@ -315,7 +325,7 @@ sub scan {
 	$self->{ttype} = -2;
 	foreach my $ttype (0 .. $#{ $self->{hook} }) {
 	    my ($j, $pl2, $j2, $sl2);         # $j  dist to candidate starting achor
-	                                      # $j2 dist to candiate ending anchor
+	                                      # $j2 dist to candidate ending anchor
 	    if (exists($self->{hook}->[$ttype]->{'begin'})) {
 		# $j - to the start of the candidate active snippet
 		($j,$pl2) = _index($self->{data}, $self->{hook}->[$ttype]->{'begin'});
@@ -339,29 +349,32 @@ sub scan {
 		unshift @args, $&;
 		$self->{ttype} = $ttype;
 		$self->{args} = \@args;
-	    }
+	      }
 	}
-
+	
 	if ($self->{ttype}==-2) {$self->{currenttoken}=$self->{data}; $self->{data}=''}
         elsif (@{$self->{args}}) {
-	    #$self->{Out} = $newOut;
-	    $self->{Out} .= substr($self->{data}, 0, $i1);
-	    #$self->{data} = $newData;
-	    $self->{data} = substr($self->{data}, $i2+$sl);
-	    $self->{prefix} = '';
-	    $self->{currenttoken} = substr($self->{data}, $i1+$pl, $i2-$i1-$pl);
-	    $self->{suffix} = '';
+	  #$self->{Out} = $newOut;
+	  $self->_process_outer( substr($self->{data}, 0, $i1) );
+	  #$self->{Out} .= substr($self->{data}, 0, $i1);
+	  ##$self->{data} = $newData;
+	  $self->{prefix} = '';
+	  $self->{currenttoken} = substr($self->{data}, $i1+$pl, $i2-$i1-$pl);
+	  $self->{suffix} = '';
+	  $self->{data} = substr($self->{data}, $i2+$sl);
 	}
 	else {
-	    $self->{Out} .= substr($self->{data}, 0, $i1); # just copy type -2
-                            # instead of returning as earlier, to
-	                    # support negative look-back for prefix
-	    $self->{prefix} = substr($self->{data}, $i1, $pl);
-	    $self->{currenttoken} = substr($self->{data}, $i1+$pl, $i2-$i1-$pl);
-	    $self->{suffix} = substr($self->{data}, $i2, $sl);
-	    $self->{data} = substr($self->{data}, $i2+$sl);
+	  # just copy type -2
+	  # instead of returning as earlier, to
+	  # support negative look-back for prefix
+	  # $self->{Out} .= substr($self->{data}, 0, $i1);
+	  $self->_process_outer( substr($self->{data}, 0, $i1) );
+	  $self->{prefix} = substr($self->{data}, $i1, $pl);
+	  $self->{currenttoken} = substr($self->{data}, $i1+$pl, $i2-$i1-$pl);
+	  $self->{suffix} = substr($self->{data}, $i2, $sl);
+	  $self->{data} = substr($self->{data}, $i2+$sl);
 	}
-    }
+      }
 
     return $self->{ttype};
 }
@@ -758,6 +771,7 @@ sub setStyle {
     
     # default
     $self->{'LineComment'} = '#';
+    $self->{'IgnoreOuter'} = '';
     $self->{hook}= [
       {begin => "\n#+\n", end=>"\n#-\n", f=>sub{return''}}, # Reserved for output
       {begin => '<?', end => '!>', f => \&evaluate },
@@ -831,7 +845,8 @@ sub setStyle {
     $self->{Style} = $s;
 }
 
-sub sfish_add_tag($$) { $::Star->add_tag(@_) }
+sub sfish_add_tag($$)     { $::Star->add_tag(@_) }
+sub sfish_ignore_outer    { $::Star->ignore_outer(@_) }
 
 # adds tags such as %slide:.* and %<slide> by adding appropriate hooks
 # eg: add_tag('slide', 'ignore')
@@ -852,6 +867,15 @@ sub add_tag {
   $self->addHook(qr/$lc$tag:(.*\n)/, $fun);
 }
 
+sub ignore_outer {
+  my $self = shift;
+  if (ref($self) ne 'Text::Starfish')
+  { unshift @_, $self; $self = $::Star; }
+  my $newignoreouter = 1;
+  $newignoreouter = $_[1] if $#_ > 0;
+  $self->{IgnoreOuter} = $newignoreouter;
+}
+
 sub add_hook {
     my $self = shift; my $ht = shift;
     my $hooks = $self->{hook}; my $hook = { ht=>$ht };
@@ -1526,7 +1550,8 @@ C<-copyhooks> Copies hooks from the Star object (C<$::Star>).  This
     the prototype object that is given to the constructor, but there
     does not seem to be need for this generalization.  More precisely,
     C<-copyhooks> copies the fields: C<Style>, C<CodePreparation>,
-    C<LineComment>, and per-component copies the array C<hook>.
+    C<LineComment>, C<IgnoreOuter>, and per-component copies
+    the array C<hook>.
 
 =head2 $o->add_tag($tag, $action)
 
@@ -1630,6 +1655,13 @@ Example:
 
  $Star->addHook(qr/^#.*\n/, 'comment');
 
+=head2 $o->ignore_outer()
+
+Sets the mode for ignoring the outer text in the replace mode.  The function
+C<sfish_ignore_outer> does the same on the default object C<Star>.
+If an argument is given, it is used to set the mode, so as a consequence
+the mode can be turned off by giving the argument ''.
+
 =head2 $o->last_update() 
 
 Or just last_update(), returns the date of the last update of the
@@ -1751,6 +1783,12 @@ and the following text is excluded:
      more lines of text excluded
      %</slide>
 
+=head2 sfish_ignore_outer()
+
+Sets the default object C<$Star> in the mode for ignoring outer text if in
+the replace mode.  If an argument is given, it is used to set the mode, so
+as a consequence the mode can be turned off with C<sfish_ignore_outer('')>.
+
 =head2 starfish_cmd I<list of file names and options>
 
 The function C<starfish_cmd> is called by the script C<starfish> with
