@@ -54,41 +54,43 @@ if (&is_module_available('CGI')) {
 &testcase(8);
 &testcase(9, 'out');
 &testcase(30); # html.sfish to html (16 ok)
-# 10
-    copy('../testfiles/9_java.out', '9_java.out');
-    #`perl -I. -- starfish -o=10_java.out -e="$slash_hack\$Starfish::HideMacros=1" 9_java.out`;
-    starfish_cmd(qw(-o=10_java.out -e=$Starfish::HideMacros=1 9_java.out));
-    ok(getfile('10_java.out'),
-	   getfile("../testfiles/10_java.out"));
-    # 11
-    copy('../testfiles/10_java.out', '10.java');
-    #`perl -I. -- starfish -o=11_java.out 10.java`;
-    starfish_cmd(qw(-o=11_java.out 10.java));
-    ok(getfile('11_java.out'),
-	   getfile("../testfiles/11_java.out"));
-    # 12
-	`echo "OSNAME | $OSNAME |"`;
-	# Skip if it is windows
-	if ($^O =~ m/MSWin/) {
-		skip('Skipped under windows...');
-	}
-	else {
-		copy('../testfiles/10_java.out', '12.java');
-		#`perl -I. -- starfish -o=12.out -mode=0444 12.java`;
-		starfish_cmd(qw(-o=12.out -mode=0444 12.java));
-		my $tmp = `ls -l 12.out|sed 's/ .*//'`;
-		`chmod u+r+w 12.out`;
-		ok($tmp, getfile("../testfiles/12.out"));
-    }
+# 10 (ok 17)
+copy('../testfiles/9_java.out', '9_java.out');
+starfish_cmd(qw(-o=10_java.out -e=$Starfish::HideMacros=1 9_java.out));
+ok(getfile('10_java.out'),
+   getfile("../testfiles/10_java.out"));
+# 11 (ok 18)
+copy('../testfiles/10_java.out', '10.java');
+starfish_cmd(qw(-o=11_java.out 10.java));
+ok(getfile('11_java.out'),
+   getfile("../testfiles/11_java.out"));
 
-    &testcase(13, 'out');
+# 12 (ok 19)
+`echo "OSNAME | $OSNAME |"`;
+# Skip if it is windows
+if ($^O =~ m/MSWin/) {
+  skip('Skipped under windows...');
+}
+else {
+  copy('../testfiles/10_java.out', '12.java');
+  #`perl -I. -- starfish -o=12.out -mode=0444 12.java`;
+  starfish_cmd(qw(-o=12.out -mode=0444 12.java));
+  #my $tmp = `ls -l 12.out|sed 's/ .*//'`;
+  #my $tmp = `stat -c %a 12.out`;
+  # checking permissions
+  my $tmp = sprintf("%o", 0777 & (stat '12.out')[2]);
+  chmod 0600, '12.out';
+  ok($tmp, "444");
+}
 
-    # 14
-    copy('../testfiles/13_java.in','14.java');
-    #`perl -I. -- starfish -o=14.out -e="$slash_hack\$Star::HideMacros=1" 14.java`;
-    starfish_cmd(qw(-o=14.out -e=$Star::HideMacros=1 14.java));
-    ok(getfile('14.out'),
-       getfile('../testfiles/14.out'));
+# (ok 20)
+&testcase(13, 'out');
+
+# 14 (ok 21)
+copy('../testfiles/13_java.in','14.java');
+#`perl -I. -- starfish -o=14.out -e="$slash_hack\$Star::HideMacros=1" 14.java`;
+starfish_cmd(qw(-o=14.out -e=$Star::HideMacros=1 14.java));
+comparefiles('../testfiles/14.out', '14.out');
 
     # 15,16
     copy('../testfiles/15.java','tmp.java');
@@ -118,26 +120,30 @@ if (&is_module_available('CGI')) {
     `$^X -I. -- starfish -e="$slash_hack\$Release=1" -o=tmp1 tmp.java`;
     okfiles('../testfiles/18.out', 'tmp1');
 
-    # 20, old 19
-	if ($^O =~ m/MSWin/) {
-		skip('Skipped under windows...');
-	}
-	else {
-		copy('../testfiles/19.html', 'tmp.html');
-	    `$^X -I. -- starfish -replace -o=tmp2 -mode=0644 tmp.html`;
-	    `ls -l tmp2|sed 's/ .*//'>tmp1`;
-		okfiles('../testfiles/19.out', 'tmp1');
-	}
+# 20, old 19 (ok 27)
+if ($^O =~ m/MSWin/) {
+  skip('Skipped under windows...');
+} else {
+  copy('../testfiles/19.html', 'tmp.html');
+  #`$^X -I. -- starfish -replace -o=tmp2 -mode=0644 tmp.html`;
+  starfish_cmd(qw(-replace -o=tmp2 -mode=0644 tmp.html));
+  # checking permissions
+  #`ls -l tmp2|sed 's/ .*//'>tmp1`;
+  my $tmp = sprintf("%o", 0777 & (stat 'tmp2')[2]);
+  ok($tmp, "644");
+}
 
-    # 21, old 20 has to be done after previous
-	if ($^O =~ m/MSWin/) {
-		skip('Skipped under windows...');
-	}
-	else {
-	    `$^X -I. -- starfish -replace -o=tmp2 tmp.html`;
-	    `ls -l tmp2|sed 's/ .*//'>tmp1`;
-	    okfiles('../testfiles/20.out', 'tmp1');
-	}
+# 21, old 20 has to be done after previous (ok 28)
+if ($^O =~ m/MSWin/) {
+  skip('Skipped under windows...');
+} else {
+  #`$^X -I. -- starfish -replace -o=tmp2 tmp.html`;
+  starfish_cmd(qw(-replace -o=tmp2 tmp.html));
+  # checking permissions
+  #`ls -l tmp2|sed 's/ .*//'>tmp1`;
+  my $tmp = sprintf("%o", 0777 & (stat 'tmp2')[2]);
+  ok($tmp, "644");
+}
 
     # 22, old 21
     copy('../testfiles/21.html','tmp2.html');
@@ -153,11 +159,15 @@ if (&is_module_available('CGI')) {
     #`$^X -I. -- starfish 24.py`;
     #okfiles('../testfiles/24.py.out', '24.py');
 
-    # 26
-    copy('../testfiles/26_include_example.html','26_include_example.html');
-    copy('../testfiles/26_include_example1.html','26_include_example1.html');
-    starfish_cmd(qw(-replace -o=26-out.html 26_include_example.html));
-    okfiles('../testfiles/26-out.html', '26-out.html');
+# 26
+copy('../testfiles/26_include_example.html','26_include_example.html');
+copy('../testfiles/26_include_example1.html','26_include_example1.html');
+starfish_cmd(qw(-replace -o=26-out.html 26_include_example.html));
+okfiles('../testfiles/26-out.html', '26-out.html');
+
+# 33
+&testcase(33, 'in:33_tex.in->33.tex -replace -o=33-lecture.tex');
+&testcase(34, 'in:33_tex.in->34.tex -replace -o=34-slides.tex');
 
 sub okfiles {
     my $f1 = shift;
@@ -184,12 +194,16 @@ sub testcase {
     my $testnum = shift;
     my ($infile, $procfile, $replace, $out, $outfile);
     my $testdir = "test-$testnum";
-    if (-d $testdir) { `rm -rfv $testdir` }
-    mkdir $testdir, 0700 or die "can't mkdir $testdir: $!";
+    if (-d $testdir) { `rm -rfv $testdir/*` }
+    else { mkdir $testdir, 0700 or die "can't mkdir $testdir: $!"; }
     chdir $testdir or die;
     my $testfilesdir = '../../testfiles';
 
-    if ( -e "$testfilesdir/$testnum.in" and $#_==-1) {
+    # example: &testcase(34, 'in:33_tex.in->34.tex -replace -o=34-slides.tex');
+    if ($#_==0 && $_[0] =~ /^in:(\S*)->(\S*) -replace -o=(\S*)$/) {
+      $infile = $1; $procfile = $2; $outfile = $replace = $3;
+    }
+    elsif ( -e "$testfilesdir/$testnum.in" and $#_==-1) {
 	$infile   = "$testnum.in";
 	$procfile = "$testnum.in";
 	$outfile  = "$testnum.out";
@@ -251,40 +265,60 @@ sub testcase {
     }
     else { die }
 
-    copy("$testfilesdir/$infile", "$procfile");
+    mycopy("$testfilesdir/$infile",  $procfile);
+    my $outExpected = "$outfile-expected";
+    my $outfile_masked = $outfile;
+    $outfile_masked =~ s/\.tex$/_tex.out/;
+    if (!-f "$testfilesdir/$outfile" && -f "$testfilesdir/$outfile_masked")
+    { mycopy("$testfilesdir/$outfile_masked", "$outfile-expected"); }
+    else
+    { mycopy("$testfilesdir/$outfile", "$outfile-expected"); }
+    my $outNew      = $procfile;
+    my @sfishArgs = ( '-e=$ver="testver"', $procfile );
     if ($replace) {
-	#`perl -I. -- starfish -e="$slash_hack\$ver=\"testver\"" -replace -o=$replace $procfile`;
-	starfish_cmd('-e=$ver="testver"', '-replace', "-o=$replace", $procfile);
-	#okfiles("$testfilesdir/$outfile", $replace);
-	comparefiles("$testfilesdir/$outfile", $replace);
+      @sfishArgs = ('-e=$ver="testver"', '-replace', "-o=$replace", $procfile);
+      $outNew = $replace;
     }
     elsif ($out) {
-	#`perl -I. -- starfish -e="$slash_hack\$ver=\"testver\"" -o=$out $procfile`;
-	starfish_cmd('-e=$ver="testver"', "-o=$out", $procfile);
-	okfiles("$testfilesdir/$outfile", $out);
-    }
-    else {
-	#`perl -I. -- starfish -e="$slash_hack\$ver=\"testver\"" $procfile`;
-	starfish_cmd('-e=$ver="testver"', $procfile);
-	okfiles("$testfilesdir/$outfile", $procfile);
+      @sfishArgs = ('-e=$ver="testver"', "-o=$out", $procfile);
+      $outNew = $out;
     }
+
+    putfile('test-description', '# CWD: '.`pwd`.
+	    "# cp $testfilesdir/$outfile $outfile-expected\n".
+	    "# perl -I. -- starfish @sfishArgs\n".
+	    "# starfish_cmd( @sfishArgs );\n".
+	    "# diff $outExpected $outNew\n");
+    starfish_cmd( @sfishArgs );
+    comparefiles($outExpected, $outNew);
+    
     chdir '..' or die;
 }
 
 sub comparefiles {
-  my $f1=shift; my $f2=shift;
-  my $fc1=getfile($f1); my $fc2=getfile($f2);
-  if ($fc1 eq $fc2) { ok(1); return; }
-  print STDERR "Difference between $f1 and $f2:\n";
-  while ($fc1 ne '' or $fc2 ne '') {
-    $fc1=~/^.*\n/; my $l1=$&; my $fc1r=$';
-    $fc2=~/^.*\n/; my $l2=$&; my $fc2r=$';
-    if ($l1 eq '' and $l2 eq '') {
+  my $f1 = shift;
+  if (!-f $f1)
+  { print STDERR "Error: file $f1 does not exist\n"; ok(0); return; }
+  my $fc1 = getfile($f1);
+  while (@_) {
+    my $f2 = shift;
+    if (!-f $f2)
+    { print STDERR "Error: file $f2 does not exist\n"; ok(0); return; }
+    my $fc2 = getfile($f2);
+    if ($fc1 eq $fc2) { next; }
+    print STDERR "CWD: ".`pwd`."Difference between $f1 and $f2:\n";
+    while ($fc1 ne '' or $fc2 ne '') {
+      $fc1=~/^.*\n/; my $l1=$&; my $fc1r=$';
+      $fc2=~/^.*\n/; my $l2=$&; my $fc2r=$';
+      if ($l1 eq '' and $l2 eq '') {
       print STDERR "$f1:$fc1", "$f2:$fc2"; last; }
-    if ($l1 ne $l2) { print STDERR "$f1:$l1", "$f2:$l2" }
-    $fc1=$fc1r; $fc2=$fc2r;
+      if ($l1 ne $l2) { print STDERR "$f1:$l1", "$f2:$l2" }
+      $fc1=$fc1r; $fc2=$fc2r;
+    }
+    ok(0);
+    return;
   }
-  ok(0);
+  ok(1);
 }
 
 sub is_module_available {
@@ -292,6 +326,12 @@ sub is_module_available {
   return '' if $@; return 1;
 }
 
+sub mycopy {
+  my $f1 = shift; my $f2 = shift;
+  if (!-f $f1) { die "File $f1 does not exist." }
+  copy($f1, $f2);
+}
+
 sub getfile($) {
     my $f = shift;
     local *F;
